@@ -18,20 +18,18 @@ OpenStack Swift client library used internally
 """
 import socket
 import re
-import requests
 import logging
 import warnings
 
-from distutils.version import StrictVersion
 from requests.exceptions import RequestException, SSLError
-from six.moves import http_client
-from six.moves.urllib.parse import quote as _quote, unquote
-from six.moves.urllib.parse import urljoin, urlparse, urlunparse
+import http.client as http_client
+from urllib.parse import quote, unquote
+from urllib.parse import urljoin, urlparse, urlunparse
 from time import sleep, time
-import six
 
 from swiftclient import version as swiftclient_version
 from swiftclient.exceptions import ClientException
+from swiftclient.requests_compat import SwiftClientRequestsSession
 from swiftclient.utils import (
     iter_wrapper, LengthWrapper, ReadableToIterable, parse_api_response,
     get_body)
@@ -48,25 +46,11 @@ USER_METADATA_TYPE = tuple('x-%s-meta-' % type_ for type_ in
 URI_PATTERN_INFO = re.compile(r'/info')
 URI_PATTERN_VERSION = re.compile(r'\/v\d+\.?\d*(\/.*)?')
 
-try:
-    from logging import NullHandler
-except ImportError:
-    # Added in Python 2.7
-    class NullHandler(logging.Handler):
-        def handle(self, record):
-            pass
-
-        def emit(self, record):
-            pass
-
-        def createLock(self):
-            self.lock = None
-
 ksexceptions = ksclient_v2 = ksclient_v3 = ksa_v3 = None
 try:
     from keystoneclient import exceptions as ksexceptions
     # prevent keystoneclient warning us that it has no log handlers
-    logging.getLogger('keystoneclient').addHandler(NullHandler())
+    logging.getLogger('keystoneclient').addHandler(logging.NullHandler())
     from keystoneclient.v2_0 import client as ksclient_v2
 except ImportError:
     pass
@@ -78,22 +62,8 @@ try:
 except ImportError:
     pass
 
-# requests version 1.2.3 try to encode headers in ascii, preventing
-# utf-8 encoded header to be 'prepared'. This also affects all
-# (or at least most) versions of requests on py3
-if StrictVersion(requests.__version__) < StrictVersion('2.0.0') \
-        or not six.PY2:
-    from requests.structures import CaseInsensitiveDict
-
-    def prepare_unicode_headers(self, headers):
-        if headers:
-            self.headers = CaseInsensitiveDict(headers)
-        else:
-            self.headers = CaseInsensitiveDict()
-    requests.models.PreparedRequest.prepare_headers = prepare_unicode_headers
-
 logger = logging.getLogger("swiftclient")
-logger.addHandler(NullHandler())
+logger.addHandler(logging.NullHandler())
 
 #: Default behaviour is to redact header values known to contain secrets,
 #: such as ``X-Auth-Key`` and ``X-Auth-Token``. Up to the first 16 chars
@@ -194,56 +164,32 @@ def http_log(args, kwargs, resp, body):
 
 
 def parse_header_string(data):
-    if not isinstance(data, (six.text_type, six.binary_type)):
+    if not isinstance(data, (str, bytes)):
         data = str(data)
-    if six.PY2:
-        if isinstance(data, six.text_type):
-            # Under Python2 requests only returns binary_type, but if we get
-            # some stray text_type input, this should prevent unquote from
-            # interpreting %-encoded data as raw code-points.
-            data = data.encode('utf8')
+    if isinstance(data, bytes):
+        # Under Python3 requests only returns text_type and tosses (!) the
+        # rest of the headers. If that ever changes, this should be a sane
+        # approach.
         try:
-            unquoted = unquote(data).decode('utf8')
+            data = data.decode('ascii')
         except UnicodeDecodeError:
-            try:
-                return data.decode('utf8')
-            except UnicodeDecodeError:
-                return quote(data).decode('utf8')
-    else:
-        if isinstance(data, six.binary_type):
-            # Under Python3 requests only returns text_type and tosses (!) the
-            # rest of the headers. If that ever changes, this should be a sane
-            # approach.
-            try:
-                data = data.decode('ascii')
-            except UnicodeDecodeError:
-                data = quote(data)
-        try:
-            unquoted = unquote(data, errors='strict')
-        except UnicodeDecodeError:
-            return data
+            data = quote(data)
+    try:
+        unquoted = unquote(data, errors='strict')
+    except UnicodeDecodeError:
+        return data
     return unquoted
 
 
-def quote(value, safe='/'):
-    """
-    Patched version of urllib.quote that encodes utf8 strings before quoting.
-    On Python 3, call directly urllib.parse.quote().
-    """
-    if six.PY3:
-        return _quote(value, safe=safe)
-    return _quote(encode_utf8(value), safe)
-
-
 def encode_utf8(value):
-    if type(value) in six.integer_types + (float, bool):
+    if type(value) in (int, float, bool):
         # As of requests 2.11.0, headers must be byte- or unicode-strings.
         # Convert some known-good types as a convenience for developers.
         # Note that we *don't* convert subclasses, as they may have overriddden
         # __str__ or __repr__.
         # See https://github.com/kennethreitz/requests/pull/3366 for more info
         value = str(value)
-    if isinstance(value, six.text_type):
+    if isinstance(value, str):
         value = value.encode('utf8')
     return value
 
@@ -255,7 +201,7 @@ def encode_meta_headers(headers):
         value = encode_utf8(value)
         header = header.lower()
 
-        if (isinstance(header, six.string_types) and
+        if (isinstance(header, str) and
                 header.startswith(USER_METADATA_TYPE)):
             header = encode_utf8(header)
 
@@ -263,12 +209,12 @@ def encode_meta_headers(headers):
     return ret
 
 
-class _ObjectBody(object):
+class _ObjectBody:
     """
     Readable and iterable object body response wrapper.
     """
 
-    def __init__(self, resp, chunk_size):
+    def __init__(self, resp, chunk_size, conn_to_close):
         """
         Wrap the underlying response
 
@@ -277,9 +223,13 @@ class _ObjectBody(object):
         """
         self.resp = resp
         self.chunk_size = chunk_size
+        self.conn_to_close = conn_to_close
 
     def read(self, length=None):
-        return self.resp.read(length)
+        buf = self.resp.read(length)
+        if length != 0 and not buf:
+            self.close()
+        return buf
 
     def __iter__(self):
         return self
@@ -295,6 +245,8 @@ class _ObjectBody(object):
 
     def close(self):
         self.resp.close()
+        if self.conn_to_close:
+            self.conn_to_close.close()
 
 
 class _RetryBody(_ObjectBody):
@@ -320,7 +272,7 @@ class _RetryBody(_ObjectBody):
         :param headers: an optional dictionary with additional headers to
                          include in the request
         """
-        super(_RetryBody, self).__init__(resp, resp_chunk_size)
+        super(_RetryBody, self).__init__(resp, resp_chunk_size, None)
         self.expected_length = int(self.resp.getheader('Content-Length'))
         self.conn = connection
         self.container = container
@@ -371,7 +323,7 @@ class _RetryBody(_ObjectBody):
         return buf
 
 
-class HTTPConnection(object):
+class HTTPConnection:
     def __init__(self, url, proxy=None, cacert=None, insecure=False,
                  cert=None, cert_key=None, ssl_compression=False,
                  default_user_agent=None, timeout=None):
@@ -406,7 +358,7 @@ class HTTPConnection(object):
         self.host = self.parsed_url.netloc
         self.port = self.parsed_url.port
         self.requests_args = {}
-        self.request_session = requests.Session()
+        self.request_session = SwiftClientRequestsSession()
         # Don't use requests's default headers
         self.request_session.headers = None
         self.resp = None
@@ -443,15 +395,6 @@ class HTTPConnection(object):
         if timeout:
             self.requests_args['timeout'] = timeout
 
-    def __del__(self):
-        """Cleanup resources other than memory"""
-        if self.request_session:
-            # The session we create must be closed to free up file descriptors
-            try:
-                self.request_session.close()
-            finally:
-                self.request_session = None
-
     def _request(self, *arg, **kwarg):
         """Final wrapper before requests call, to be patched in tests"""
         return self.request_session.request(*arg, **kwarg)
@@ -489,12 +432,12 @@ class HTTPConnection(object):
         old_getheader = self.resp.raw.getheader
 
         def _decode_header(string):
-            if string is None or six.PY2:
+            if string is None:
                 return string
             return string.encode('iso-8859-1').decode('utf-8')
 
         def _encode_header(string):
-            if string is None or six.PY2:
+            if string is None:
                 return string
             return string.encode('utf-8').decode('iso-8859-1')
 
@@ -513,7 +456,7 @@ class HTTPConnection(object):
                 # urllib3's connection pool. This will reduce the number of
                 # log messages seen in bug #1341777. This does not actually
                 # close a socket. It will also prevent people from being
-                # mislead as to the cause of a bug as in bug #1424732.
+                # misled as to the cause of a bug as in bug #1424732.
                 self.resp.close()
             return chunk
 
@@ -843,8 +786,10 @@ def get_account(url, token, marker=None, limit=None, prefix=None,
     if headers:
         req_headers.update(headers)
 
+    close_conn = False
     if not http_conn:
         http_conn = http_connection(url)
+        close_conn = True
     if full_listing:
         rv = get_account(url, token, marker, limit, prefix, end_marker,
                          http_conn, headers=req_headers, delimiter=delimiter)
@@ -874,6 +819,8 @@ def get_account(url, token, marker=None, limit=None, prefix=None,
     conn.request(method, full_path, '', req_headers)
     resp = conn.getresponse()
     body = resp.read()
+    if close_conn:
+        conn.close()
     http_log(("%s?%s" % (url, qs), method,), {'headers': req_headers},
              resp, body)
 
@@ -900,10 +847,12 @@ def head_account(url, token, http_conn=None, headers=None,
               be lowercase)
     :raises ClientException: HTTP HEAD request failed
     """
+    close_conn = False
     if http_conn:
         parsed, conn = http_conn
     else:
         parsed, conn = http_connection(url)
+        close_conn = True
     method = "HEAD"
     req_headers = {'X-Auth-Token': token}
     if service_token:
@@ -914,6 +863,8 @@ def head_account(url, token, http_conn=None, headers=None,
     conn.request(method, parsed.path, '', req_headers)
     resp = conn.getresponse()
     body = resp.read()
+    if close_conn:
+        conn.close()
     http_log((url, method,), {'headers': req_headers}, resp, body)
     if resp.status < 200 or resp.status >= 300:
         raise ClientException.from_response(resp, 'Account HEAD failed', body)
@@ -939,10 +890,12 @@ def post_account(url, token, headers, http_conn=None, response_dict=None,
     :raises ClientException: HTTP POST request failed
     :returns: resp_headers, body
     """
+    close_conn = False
     if http_conn:
         parsed, conn = http_conn
     else:
         parsed, conn = http_connection(url)
+        close_conn = True
     method = 'POST'
     path = parsed.path
     if query_string:
@@ -955,6 +908,8 @@ def post_account(url, token, headers, http_conn=None, response_dict=None,
     conn.request(method, path, data, req_headers)
     resp = conn.getresponse()
     body = resp.read()
+    if close_conn:
+        conn.close()
     http_log((url, method,), {'headers': req_headers}, resp, body)
 
     store_response(resp, response_dict)
@@ -996,8 +951,10 @@ def get_container(url, token, container, marker=None, limit=None,
               headers will be a dict and all header names will be lowercase.
     :raises ClientException: HTTP GET request failed
     """
+    close_conn = False
     if not http_conn:
         http_conn = http_connection(url)
+        close_conn = True
     if full_listing:
         rv = get_container(url, token, container, marker, limit, prefix,
                            delimiter, end_marker, version_marker, path=path,
@@ -1046,6 +1003,8 @@ def get_container(url, token, container, marker=None, limit=None,
     conn.request(method, '%s?%s' % (cont_path, qs), '', req_headers)
     resp = conn.getresponse()
     body = resp.read()
+    if close_conn:
+        conn.close()
     http_log(('%(url)s%(cont_path)s?%(qs)s' %
               {'url': url.replace(parsed.path, ''),
                'cont_path': cont_path,
@@ -1076,10 +1035,12 @@ def head_container(url, token, container, http_conn=None, headers=None,
               be lowercase)
     :raises ClientException: HTTP HEAD request failed
     """
+    close_conn = False
     if http_conn:
         parsed, conn = http_conn
     else:
         parsed, conn = http_connection(url)
+        close_conn = True
     path = '%s/%s' % (parsed.path, quote(container))
     method = 'HEAD'
     req_headers = {'X-Auth-Token': token}
@@ -1090,6 +1051,8 @@ def head_container(url, token, container, http_conn=None, headers=None,
     conn.request(method, path, '', req_headers)
     resp = conn.getresponse()
     body = resp.read()
+    if close_conn:
+        conn.close()
     http_log(('%s%s' % (url.replace(parsed.path, ''), path), method,),
              {'headers': req_headers}, resp, body)
 
@@ -1117,10 +1080,12 @@ def put_container(url, token, container, headers=None, http_conn=None,
     :param query_string: if set will be appended with '?' to generated path
     :raises ClientException: HTTP PUT request failed
     """
+    close_conn = False
     if http_conn:
         parsed, conn = http_conn
     else:
         parsed, conn = http_connection(url)
+        close_conn = True
     path = '%s/%s' % (parsed.path, quote(container))
     method = 'PUT'
     req_headers = {'X-Auth-Token': token}
@@ -1135,6 +1100,8 @@ def put_container(url, token, container, headers=None, http_conn=None,
     conn.request(method, path, '', req_headers)
     resp = conn.getresponse()
     body = resp.read()
+    if close_conn:
+        conn.close()
 
     store_response(resp, response_dict)
 
@@ -1160,10 +1127,12 @@ def post_container(url, token, container, headers, http_conn=None,
     :param service_token: service auth token
     :raises ClientException: HTTP POST request failed
     """
+    close_conn = False
     if http_conn:
         parsed, conn = http_conn
     else:
         parsed, conn = http_connection(url)
+        close_conn = True
     path = '%s/%s' % (parsed.path, quote(container))
     method = 'POST'
     req_headers = {'X-Auth-Token': token}
@@ -1176,6 +1145,8 @@ def post_container(url, token, container, headers, http_conn=None,
     conn.request(method, path, '', req_headers)
     resp = conn.getresponse()
     body = resp.read()
+    if close_conn:
+        conn.close()
     http_log(('%s%s' % (url.replace(parsed.path, ''), path), method,),
              {'headers': req_headers}, resp, body)
 
@@ -1204,10 +1175,12 @@ def delete_container(url, token, container, http_conn=None,
     :param headers: additional headers to include in the request
     :raises ClientException: HTTP DELETE request failed
     """
+    close_conn = False
     if http_conn:
         parsed, conn = http_conn
     else:
         parsed, conn = http_connection(url)
+        close_conn = True
     path = '%s/%s' % (parsed.path, quote(container))
     if headers:
         headers = dict(headers)
@@ -1223,6 +1196,8 @@ def delete_container(url, token, container, http_conn=None,
     conn.request(method, path, '', headers)
     resp = conn.getresponse()
     body = resp.read()
+    if close_conn:
+        conn.close()
     http_log(('%s%s' % (url.replace(parsed.path, ''), path), method,),
              {'headers': headers}, resp, body)
 
@@ -1244,7 +1219,8 @@ def get_object(url, token, container, name, http_conn=None,
     :param container: container name that the object is in
     :param name: object name to get
     :param http_conn: a tuple of (parsed url, HTTPConnection object),
-                      (If None, it will create the conn object)
+                      (If None, it will create the conn object and close it
+                      after all content is read)
     :param resp_chunk_size: if defined, chunk size of data to read. NOTE: If
                             you specify a resp_chunk_size you must fully read
                             the object's contents before making another
@@ -1259,10 +1235,12 @@ def get_object(url, token, container, name, http_conn=None,
               headers will be a dict and all header names will be lowercase.
     :raises ClientException: HTTP GET request failed
     """
+    close_conn = False
     if http_conn:
         parsed, conn = http_conn
     else:
         parsed, conn = http_connection(url)
+        close_conn = True
     path = '%s/%s/%s' % (parsed.path, quote(container), quote(name))
     if query_string:
         path += '?' + query_string
@@ -1285,9 +1263,12 @@ def get_object(url, token, container, name, http_conn=None,
                  {'headers': headers}, resp, body)
         raise ClientException.from_response(resp, 'Object GET failed', body)
     if resp_chunk_size:
-        object_body = _ObjectBody(resp, resp_chunk_size)
+        object_body = _ObjectBody(resp, resp_chunk_size,
+                                  conn_to_close=conn if close_conn else None)
     else:
         object_body = resp.read()
+        if close_conn:
+            conn.close()
     http_log(('%s%s' % (url.replace(parsed.path, ''), path), method,),
              {'headers': headers}, resp, None)
 
@@ -1311,10 +1292,12 @@ def head_object(url, token, container, name, http_conn=None,
               be lowercase)
     :raises ClientException: HTTP HEAD request failed
     """
+    close_conn = False
     if http_conn:
         parsed, conn = http_conn
     else:
         parsed, conn = http_connection(url)
+        close_conn = True
     path = '%s/%s/%s' % (parsed.path, quote(container), quote(name))
     if query_string:
         path += '?' + query_string
@@ -1329,6 +1312,8 @@ def head_object(url, token, container, name, http_conn=None,
     conn.request(method, path, '', headers)
     resp = conn.getresponse()
     body = resp.read()
+    if close_conn:
+        conn.close()
     http_log(('%s%s' % (url.replace(parsed.path, ''), path), method,),
              {'headers': headers}, resp, body)
     if resp.status < 200 or resp.status >= 300:
@@ -1378,10 +1363,12 @@ def put_object(url, token=None, container=None, name=None, contents=None,
     :returns: etag
     :raises ClientException: HTTP PUT request failed
     """
+    close_conn = False
     if http_conn:
         parsed, conn = http_conn
     else:
         parsed, conn = http_connection(url, proxy=proxy)
+        close_conn = True
     path = parsed.path
     if container:
         path = '%s/%s' % (path.rstrip('/'), quote(container))
@@ -1407,11 +1394,6 @@ def put_object(url, token=None, container=None, name=None, contents=None,
                 content_length = int(v)
     if content_type is not None:
         headers['Content-Type'] = content_type
-    elif 'Content-Type' not in headers:
-        if StrictVersion(requests.__version__) < StrictVersion('2.4.0'):
-            # python-requests sets application/x-www-form-urlencoded otherwise
-            # if using python3.
-            headers['Content-Type'] = ''
     if not contents:
         headers['Content-Length'] = '0'
 
@@ -1434,12 +1416,14 @@ def put_object(url, token=None, container=None, name=None, contents=None,
             warnings.warn(warn_msg, stacklevel=2)
         # Match requests's is_stream test
         if hasattr(contents, '__iter__') and not isinstance(contents, (
-                six.text_type, six.binary_type, list, tuple, dict)):
+                str, bytes, list, tuple, dict)):
             contents = iter_wrapper(contents)
         conn.request('PUT', path, contents, headers)
 
     resp = conn.getresponse()
     body = resp.read()
+    if close_conn:
+        conn.close()
     http_log(('%s%s' % (url.replace(parsed.path, ''), path), 'PUT',),
              {'headers': headers}, resp, body)
 
@@ -1469,10 +1453,12 @@ def post_object(url, token, container, name, headers, http_conn=None,
     :param service_token: service auth token
     :raises ClientException: HTTP POST request failed
     """
+    close_conn = False
     if http_conn:
         parsed, conn = http_conn
     else:
         parsed, conn = http_connection(url)
+        close_conn = True
     path = '%s/%s/%s' % (parsed.path, quote(container), quote(name))
     req_headers = {'X-Auth-Token': token}
     if service_token:
@@ -1482,6 +1468,8 @@ def post_object(url, token, container, name, headers, http_conn=None,
     conn.request('POST', path, '', req_headers)
     resp = conn.getresponse()
     body = resp.read()
+    if close_conn:
+        conn.close()
     http_log(('%s%s' % (url.replace(parsed.path, ''), path), 'POST',),
              {'headers': req_headers}, resp, body)
 
@@ -1514,10 +1502,12 @@ def copy_object(url, token, container, name, destination=None,
     :param service_token: service auth token
     :raises ClientException: HTTP COPY request failed
     """
+    close_conn = False
     if http_conn:
         parsed, conn = http_conn
     else:
         parsed, conn = http_connection(url)
+        close_conn = True
 
     path = parsed.path
     container = quote(container)
@@ -1546,6 +1536,8 @@ def copy_object(url, token, container, name, destination=None,
     conn.request('COPY', path, '', headers)
     resp = conn.getresponse()
     body = resp.read()
+    if close_conn:
+        conn.close()
     http_log(('%s%s' % (url.replace(parsed.path, ''), path), 'COPY',),
              {'headers': headers}, resp, body)
 
@@ -1578,10 +1570,12 @@ def delete_object(url, token=None, container=None, name=None, http_conn=None,
     :param service_token: service auth token
     :raises ClientException: HTTP DELETE request failed
     """
+    close_conn = False
     if http_conn:
         parsed, conn = http_conn
     else:
         parsed, conn = http_connection(url, proxy=proxy)
+        close_conn = True
     path = parsed.path
     if container:
         path = '%s/%s' % (path.rstrip('/'), quote(container))
@@ -1600,6 +1594,8 @@ def delete_object(url, token=None, container=None, name=None, http_conn=None,
     conn.request('DELETE', path, '', headers)
     resp = conn.getresponse()
     body = resp.read()
+    if close_conn:
+        conn.close()
     http_log(('%s%s' % (url.replace(parsed.path, ''), path), 'DELETE',),
              {'headers': headers}, resp, body)
 
@@ -1630,7 +1626,7 @@ def get_capabilities(http_conn):
     return parse_api_response(resp_headers, body)
 
 
-class Connection(object):
+class Connection:
 
     """
     Convenience class to make requests that will also retry the request
@@ -1650,7 +1646,7 @@ class Connection(object):
                  starting_backoff=1, max_backoff=64, tenant_name=None,
                  os_options=None, auth_version="1", cacert=None,
                  insecure=False, cert=None, cert_key=None,
-                 ssl_compression=True, retry_on_ratelimit=False,
+                 ssl_compression=True, retry_on_ratelimit=True,
                  timeout=None, session=None, force_auth_retry=False):
         """
         :param authurl: authentication URL
@@ -1682,9 +1678,9 @@ class Connection(object):
                                 will be made. This may provide a performance
                                 increase for https upload/download operations.
         :param retry_on_ratelimit: by default, a ratelimited connection will
-                                   raise an exception to the caller. Setting
-                                   this parameter to True will cause a retry
-                                   after a backoff.
+                                   retry after a backoff. Setting this
+                                   parameter to False will cause an exception
+                                   to be raised to the caller.
         :param timeout: The connect timeout for the HTTP connection.
         :param session: A keystoneauth session object.
         :param force_auth_retry: reset auth info even if client got unexpected
@@ -1829,7 +1825,7 @@ class Connection(object):
                     self.http_conn = None
                 elif 500 <= err.http_status <= 599:
                     pass
-                elif self.retry_on_ratelimit and err.http_status == 498:
+                elif self.retry_on_ratelimit and err.http_status in (498, 429):
                     pass
                 else:
                     raise
